#include <Wire.h>
#include <SimpleDHT.h>
#include <LiquidCrystal_I2C.h>

#define PIN_GAS 	A3
#define BUZZER_PIN	13
#define BUTTON_PIN	3
#define SMOKE 0
#define DHTPIN 2
volatile bool fireDetected = false;

SimpleDHT11 dht11(DHTPIN);

LiquidCrystal_I2C lcd(0x27, 16, 2); 


void USART0_init()
{
  /* seteaza baud rate la 9600 */
  UBRR0 = 103;

  /* porneste transmitatorul */
  UCSR0B = (1 << TXEN0) | (1 << RXEN0);

  /* seteaza formatul frame-ului: 8 biti de date, 1 bit de stop, fara paritate */
  UCSR0C &= ~(1 << USBS0);
  UCSR0C |= (3 << UCSZ00);
}

/*
   Functie ce transmite un caracter prin USART

   @param data - caracterul de transmis
*/
void USART0_transmit(char data)
{
  /* asteapta pana bufferul e gol */
  while (!(UCSR0A & (1 << UDRE0)))
    ;

  /* pune datele in buffer; transmisia va porni automat in urma scrierii */
  UDR0 = data;
}

/*
   Functie ce primeste un caracter prin USART

   @return - caracterul primit
*/
char USART0_receive()
{
  /* asteapta cat timp bufferul e gol */
  while (!(UCSR0A & (1 << RXC0)))
    ;

  /* returneaza datele din buffer */
  return UDR0;
}

/*
   Functie ce transmite un sir de caractere prin USART

   @param data - sirul (terminat cu '\0') de transmis
*/
void USART0_print(const char *data)
{
  while (*data != '\0')
    USART0_transmit(*data++);
}

void buttonPressed() {
  if(!fireDetected) {
    USART0_print("is pressed\n");
    fireDetected = true;
  } else {
    USART0_print("is not pressed\n");
    fireDetected = false;
  }
  
}

void setup()
{
  USART0_init();
  lcd.init();
  lcd.backlight();
  pinMode(BUZZER_PIN, OUTPUT);
  pinMode(BUTTON_PIN, INPUT_PULLUP);

  ADMUX = 0;
  /* ADC1 - channel 1 */
  ADMUX |= (1 << MUX0);
  /* AVCC with external capacitor at AREF pin */
  ADMUX |= (1 << REFS0);

  ADCSRA = 0;
  /* set prescaler at 128 */
  ADCSRA |= (7 << ADPS0);
  /* enable ADC */
  ADCSRA |= (1 << ADEN);

  EICRA |= (1 << ISC11);  // Set falling edge trigger 
  EIMSK |= (1 << INT1);   // Enable external interrupt INT0

  sei();  // Enable global interrupts
}

ISR(INT1_vect) {
  buttonPressed();
}

int USART0_available()
{
  return (UCSR0A & (1 << RXC0)) ? 1 : 0;
}

void loop()
{
  //delay(1000);
  while(USART0_available()){
    // Read the received word until newline character ('\n') is encountered
    char receivedChar = USART0_receive();
    if(receivedChar == 'i') {
      fireDetected = true;
    } else if (receivedChar == 's'){
      fireDetected = false;
    }

  }

  lcd.clear();
  lcd.setCursor(0, 0);  // Setează cursorul pe prima poziție (linia 0, coloana 0)
  byte temperature = 0;
  byte humidity = 0;
  int err = dht11.read(&temperature, &humidity, NULL);
  // int sensorValue=analogRead(SMOKE);
  /* start conversion */
  ADCSRA |= (1 << ADSC);
  /* wait until conversion is complete */
  while (!(ADCSRA & (1 << ADIF)));
  uint16_t sensorValue = ADC;

  USART0_print("Gas sensor: \n");
  char buffer[10];
  USART0_print(itoa(sensorValue, buffer, 10));
  USART0_print("\n");

  lcd.setCursor(0,1);
  lcd.print(temperature);
  lcd.print(" degrees C ");


  if(temperature >= 30 || fireDetected || sensorValue > 400)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("incendiuu!!");
    tone(BUZZER_PIN, 1000,200); 
    delay(500); 
    noTone(BUZZER_PIN);    
    delay(500);        
  }
 
  delay(1000);
}